@@ -1,17 +1,18 @@
-import codecs
-import json
 import os
 import re
 import uuid
+import json
+import codecs
+import shutil
 from pathlib import Path
 from urllib.parse import urlparse
-
 from lyrebird.log import get_logger
 from jinja2 import Template
 from lyrebird.application import config
+from lyrebird.mock.handlers import snapshot_helper
+
 
 PROP_FILE_NAME = '.lyrebird_prop'
-
 logger = get_logger()
 
 
@@ -34,6 +35,7 @@ class DataManager:
         self.clipboard = None
         self.save_to_group_id = None
         self.tmp_group = {'id': 'tmp_group', 'type': 'group', 'name': 'tmp-group', 'children': []}
+        self.snapshot_helper = snapshot_helper.SnapshotHelper()
 
     def set_root(self, root_path):
         """
@@ -347,41 +349,56 @@ class DataManager:
             'node': _node
         }
 
-    def paste(self, parent_id):
+    def import_(self, node):
+        self.clipboard = {
+            'type': 'import',
+            'id': node["id"],
+            'node': node
+        }
+
+    def paste(self, parent_id, **kwargs):
         if not self.clipboard:
             raise NoneClipbord
         _parent_node = self.id_map.get(parent_id)
         _node = self.clipboard['node']
         if not _parent_node:
             raise IDNotFound(parent_id)
+        if not _parent_node.get('children'):
+            _parent_node['children'] = []
         if self.clipboard['type'] == 'cut':
             _origin_parent = self.id_map.get(_node['parent_id'])
             _origin_parent['children'].remove(_node)
             _parent_node['children'].insert(0, _node)
             _node['parent_id'] = parent_id
         elif self.clipboard['type'] == 'copy':
-            self._copy_node(_parent_node, _node)
+            self._copy_node(_parent_node, _node, **kwargs)
+        elif self.clipboard['type'] == 'import':
+            self._copy_node(_parent_node, _node, **kwargs)
         self._save_prop()
 
-    def _copy_node(self, parent_node, node):
+    def _copy_node(self, parent_node, node, **kwargs):
         new_node = {}
         new_node.update(node)
         new_node['id'] = str(uuid.uuid4())
         new_node['parent_id'] = parent_node['id']
         # Add to target node
+        if not parent_node.get('children'):
+            parent_node['children'] = []
         parent_node['children'].insert(0, new_node)
         # Register ID
         self.id_map[new_node['id']] = new_node
         if new_node['type'] == 'group':
             new_node['children'] = []
             for child in node['children']:
-                self._copy_node(new_node, child)
+                self._copy_node(new_node, child, **kwargs)
         elif new_node['type'] == 'data':
-            self._copy_file(new_node, node)
+            self._copy_file(new_node, node, **kwargs)
 
-    def _copy_file(self, target_data_node, data_node):
+    def _copy_file(self, target_data_node, data_node, **kwargs):
         _id = data_node['id']
         origin_file_path = self.root_path / _id
+        if kwargs.get('custom_input_file_path'):
+            origin_file_path = f'{kwargs.get("custom_input_file_path")}/{_id}'
         new_file_id = target_data_node['id']
         new_file_path = self.root_path / new_file_id
         with codecs.open(origin_file_path, 'r') as inputfile, codecs.open(new_file_path, 'w') as outputfile:
@@ -544,6 +561,54 @@ class DataManager:
             f.write(data_str)
         self._save_prop()
 
+    # -----
+    # Snapshot
+    # -----
+
+    def _write_prop_to_custom_path(self, outfile_path, node):
+        prop_str = PropWriter().parse(node)
+
+        prop_file = outfile_path / PROP_FILE_NAME
+        with codecs.open(prop_file, "w") as f:
+            f.write(prop_str)
+
+    def _write_file_to_custom_path(self, outfile_path, file_content):
+        with codecs.open(outfile_path / file_content['id'], "w") as f:
+            f.write(json.dumps(file_content, ensure_ascii=False))
+
+    def import_snapshot(self, parent_id):
+        snapshot_path = self.snapshot_helper.get_snapshot_path()
+        self.snapshot_helper.save_compressed_file(snapshot_path)
+        self.snapshot_helper.decompress_snapshot(f"{snapshot_path}.lb", f"{snapshot_path}-decompressed")
+        if not Path(f"{snapshot_path}-decompressed/{PROP_FILE_NAME}").exists():
+            raise LyrebirdPropNotExists
+        with codecs.open(f"{snapshot_path}-decompressed/{PROP_FILE_NAME}") as f:
+            _prop = json.load(f)
+        self.import_(node=_prop)
+        self.paste(parent_id=parent_id, custom_input_file_path=f"{snapshot_path}-decompressed")
+
+    def export_snapshot_from_event(self, event_json):
+        snapshot_path = self.snapshot_helper.get_snapshot_path()
+        if not event_json.get("snapshot") or not event_json.get("events"):
+            raise SnapshotEventNotInCorrectFormat
+        _prop = event_json.get("snapshot")
+        self._write_prop_to_custom_path(snapshot_path, _prop)
+        for mock_data in event_json.get("events"):
+            self._write_file_to_custom_path(snapshot_path, mock_data)
+        self.snapshot_helper.compress_snapshot(snapshot_path, snapshot_path)
+        return f"{snapshot_path}.lb"
+
+    def export_snapshot_from_dm(self, node_id):
+        snapshot_path = self.snapshot_helper.get_snapshot_path()
+        _prop = self.id_map.get(node_id)
+        self._write_prop_to_custom_path(snapshot_path, _prop)
+        data_id_map = {}
+        self.snapshot_helper.get_data_id_map(_prop, data_id_map)
+        for mock_data_id in data_id_map:
+            shutil.copy(self.root_path / mock_data_id, snapshot_path / mock_data_id)
+        self.snapshot_helper.compress_snapshot(snapshot_path, snapshot_path)
+        return f"{snapshot_path}.lb"
+
 
 # -----------------
 # Exceptions
@@ -589,6 +654,22 @@ class DumpPropError(Exception):
     pass
 
 
+class NonePropFile(Exception):
+    pass
+
+
+class TooMuchPropFile(Exception):
+    pass
+
+
+class NodeExist(Exception):
+    pass
+
+
+class SnapshotEventNotInCorrectFormat(Exception):
+    pass
+
+
 class PropWriter:
 
     def __init__(self):
